/*
 * FRAM_MB85RS4MT_SPI.c
 *  Created on: 20.03.2015 as FRAM_RS2MT_SPI.c
 *      Author: DL9MWE
 *  21.04.2019
 */


#include "main.h"

void vFRAM_RS4MT_SPI_init() {
    vSPI_init();
    // Chip Select ist Ausgang und wird auf 1 gesetzt
    PORT_SPI_SEL &= ~(PORTFRAM_CS1_PIN+PORTFRAM_CS2_PIN);
    PORT_SPI_SEL2 &= ~(PORTFRAM_CS1_PIN+PORTFRAM_CS2_PIN);
    PORT_SPI_DIR |= (PORTFRAM_CS1_PIN+PORTFRAM_CS2_PIN);
    PORT_SPI_OUT |= (PORTFRAM_CS1_PIN+PORTFRAM_CS2_PIN);
}


unsigned long ulFRAM_RS4MT_writebytes(unsigned long addr, unsigned char *txData, unsigned int anzahl) {
    unsigned int SPI_CS_pin;
    unsigned long wr_addr;      // tatschliche Schreibadresse im FRAM
    wr_addr = addr;
    if(addr <= FRAM_maxAddr) {
        SPI_CS_pin = PORTFRAM_CS1_PIN;
    }
    else {
        SPI_CS_pin = PORTFRAM_CS2_PIN;
        wr_addr = addr-FRAM_maxAddr-1;
    }
    PORT_SPI_OUT &= ~SPI_CS_pin;           // _CS auf 0
    ucSPI_Byte(FRAM_WREN);                 // enable Write
    PORT_SPI_OUT |= SPI_CS_pin;            // _CS auf 1

    PORT_SPI_OUT &= ~SPI_CS_pin;           // _CS auf 0
    ucSPI_Byte(FRAM_WRITE);
    ucSPI_Byte((unsigned char)(wr_addr>>16));
    ucSPI_Byte((unsigned char)(wr_addr>>8));
    ucSPI_Byte((unsigned char) wr_addr);

    unsigned int index=0;
    while(index < anzahl) {
        ucSPI_Byte(txData[index++]);
        ++addr;
        if(++wr_addr > FRAM_maxAddr) {            // Speicherende des FRAMs erreicht?
            PORT_SPI_OUT |= SPI_CS_pin;           // _CS auf 1
            if(SPI_CS_pin == PORTFRAM_CS2_PIN) {  // Ende des zweiten FRAMs
                addr = 0UL;                       // Fehler: Speicherberlauf!
                break;
            }
            // sonst im zweiten FRAM weiterschreiben
            addr = ulFRAM_RS4MT_writebytes(wr_addr, &txData[index], anzahl-index);
            break;
        }
    }
    PORT_SPI_OUT |= PORTFRAM_CS1_PIN + PORTFRAM_CS2_PIN;    // beide _CS auf 1
    return addr;
}

unsigned long ulFRAM_RS4MT_readbytes(unsigned long addr, unsigned char *rxData, unsigned int anzahl) {
    unsigned int SPI_CS_pin;
    unsigned long rd_addr;      // tatschliche Leseadresse im FRAM
    rd_addr = addr;

    if(addr <= FRAM_maxAddr) {
        SPI_CS_pin = PORTFRAM_CS1_PIN;
    }
    else {
        SPI_CS_pin = PORTFRAM_CS2_PIN;
        rd_addr = addr-FRAM_maxAddr-1;
    }
    PORT_SPI_OUT &= ~SPI_CS_pin;           // _CS auf 0
    ucSPI_Byte(FRAM_READ);
    ucSPI_Byte((unsigned char)(rd_addr>>16));
    ucSPI_Byte((unsigned char)(rd_addr>>8));
    ucSPI_Byte((unsigned char) rd_addr);

    unsigned int index=0;
    while(index < anzahl) {
        rxData[index++] = ucSPI_Byte(0x00);
        ++addr;
        if(++rd_addr > FRAM_maxAddr) {                 // Speicherende des FRAMs erreicht?
            PORT_SPI_OUT |= SPI_CS_pin;             // _CS auf 1
            if(SPI_CS_pin == PORTFRAM_CS2_PIN) {    // Ende des zweiten FRAMs
                addr = 0UL;                         // Fehler: Speicherberlauf!
                break;
            }
            // sonst im zweiten FRAM weiterlesen
            addr = ulFRAM_RS4MT_readbytes(rd_addr, &rxData[index], anzahl-index);
            break;
        }
    }
    PORT_SPI_OUT |= PORTFRAM_CS1_PIN + PORTFRAM_CS2_PIN;            // beide _CS auf 1
    return addr;
}

// Ende FRAM_MB85RS4MT_SPI.c
