/**
 * main.c
 * Test des FRAMs MB85RS4MT (SPI) 4MBit ergeben 512kByte pro Chip
 * Zwei FRAMs werden verwendet, also zusammen 1MByte; der Adressraum geht somit von 0 bis 2*7FFFFh
 * Der Prozessor ist ein MSP430G2553
 * 22.4.2019 DL9MWE
 */
#include "main.h"

int main(void)
{
    unsigned long FRAM_txaddr;  // Speicheradresse in den FRAMs frs Schreiben, Wertebereich 0-2*7FFFF
    unsigned long FRAM_rxaddr;  // Speicheradresse in den FRAMs frs Lesen, Wertebereich dito
    unsigned char *txData1 = "1234567890";
    unsigned char *txData2 = "ABCDEFGHIJ";
    unsigned char *txData3 = "KLMNOPQRST";
    unsigned char rxData[10];   // Array fr die zurckgelesenen Daten

    WDTCTL = WDTPW | WDTHOLD;   // stop watchdog timer
    vCPU_init();
    vFRAM_RS4MT_SPI_init();

    // Test des ersten FRAM-Chips
    FRAM_txaddr = 0UL;
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData1, 10);
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData2, 10);
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData3, 10);
    FRAM_rxaddr = 0UL;
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);

    // Test des zweiten FRAM-Chips
    FRAM_txaddr = FRAM_maxAddr+1;
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData1, 10);
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData2, 10);
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData3, 10);
    FRAM_rxaddr = FRAM_maxAddr+1;
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);

    // Test des berlaufs zwischen erstem und zweiten FRAM
    FRAM_txaddr = FRAM_maxAddr-5;
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData1, 10);
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData2, 10);
    FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData3, 10);
    FRAM_rxaddr = FRAM_maxAddr-5;
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);
    FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);

    // Test des Erreichens des Speicherendes
    FRAM_txaddr = 2*FRAM_maxAddr-12;
    if((FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData1, 10)) != 0UL)
        if((FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData2, 10)) != 0UL)
            FRAM_txaddr = ulFRAM_RS4MT_writebytes(FRAM_txaddr, txData3, 10);

    FRAM_rxaddr = 2*FRAM_maxAddr-12;
    if((FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10)) != 0UL)
        if((FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10)) != 0UL)
            FRAM_rxaddr = ulFRAM_RS4MT_readbytes(FRAM_rxaddr, rxData, 10);

    return 0;
}

void vCPU_init(void) {
    // fr MCLK wird der interne Oszillator wird verwendet  => Set DCO to 16 MHz
    DCOCTL = CALDCO_16MHZ;
    BCSCTL1 = CALBC1_16MHZ;
    BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;
}

// msec Millisekunden warten
void _delay_ms(unsigned int msec) {
  volatile unsigned int count;
    while(msec--) {
        count = 1600;   // passt bei 16MHz aus dem internen Oszillator DCO
        while(count) {
            --count;
        }
    }
}

// usec Mikrosekunden warten (ungefhr...)
void _delay_us(volatile unsigned int usec) {
    while(usec--) {
        _NOP();
        _NOP();
        _NOP();
        _NOP();
    }
}

// Ende main.c
